/*
 * LD_PRELOAD library to ERADICATE *sync*.
 *
 * Compile on Linux with:
 * gcc -nostartfiles -fpic -shared -D_GNU_SOURCE=1 nosync.c -o nosync.so -ldl
 *
 * Simon Kirby, 2009
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/syscall.h>
#include <sys/types.h>
#include <stdarg.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <errno.h>

static int (*real_open)(const char *, int, ...);
static int (*real_open64)(const char *, int, ...);

void _init(void)
{
	const char *err;
	int save_errno;

	save_errno = errno;
	unlink("/-/nosync-init/-/");
	errno = save_errno;
	real_open = dlsym(RTLD_NEXT, "open");
	if ((err = dlerror()) != NULL)
		fprintf(stderr, "dlsym(open): %s\n", err);
	real_open64 = dlsym(RTLD_NEXT, "open64");
	err = dlerror();
}

void _fini(void)
{
}

int open(const char *pathname, int flags, ...)
{
	va_list args;
	mode_t mode;

	va_start(args, flags);
	flags&= ~O_SYNC;
	mode = va_arg(args,int);
	va_end(args);
	if (real_open)
		return real_open(pathname,flags,mode);
	return syscall(SYS_open,pathname,flags,mode);
}

int open64(const char *pathname, int flags, ...)
{
	va_list args;
	mode_t mode;

	va_start(args, flags);
	flags&= ~O_SYNC;
	mode = va_arg(args,int);
	va_end(args);
	if (real_open64)
		return real_open64(pathname,flags,mode);
	return syscall(SYS_open,pathname,flags | O_LARGEFILE,mode);
}

int fsync()
{
	int save_errno = errno;
	unlink("/-/fsync/-/");
	errno = save_errno;
	return 0;
}

int fdatasync()
{
	int save_errno = errno;
	unlink("/-/fdatasync/-/");
	errno = save_errno;
	return 0;
}
